@@ -24,19 +24,26 @@
 #include "cookie-jar-manager.h"
 #include "debug.h"
 #include "dialog.h"
+#include "http-warning.h"
 #include "i18n.h"
 
 #include <QDBusArgument>
 #include <QDesktopServices>
+#include <QIcon>
 #include <QLabel>
 #include <QNetworkCookie>
+#include <QNetworkReply>
 #include <QNetworkRequest>
+#include <QPixmap>
 #include <QPointer>
 #include <QProgressBar>
 #include <QPushButton>
 #include <QRegExp>
 #include <QSettings>
+#include <QSslError>
 #include <QStackedLayout>
+#include <QStatusBar>
+#include <QTimer>
 #include <QVBoxLayout>
 #include <QWebElement>
 #include <QWebFrame>
@@ -67,6 +74,7 @@ static const QString valueAlwaysOff = QString("alwaysOff");
 /* Additional session-data keys we support. */
 static const QString keyCookies = QString("Cookies");
 static const QString keyAllowedSchemes = QString("AllowedSchemes");
+static const QString keyIgnoreSslErrors = QString("IgnoreSslErrors");
 
 class WebPage: public QWebPage
 {
@@ -152,6 +160,8 @@ private:
 };
 
 bool WebPage::urlIsBlocked(QUrl url) const {
+    if (url == QUrl("about:blank")) return false;
+
     if (!m_allowedSchemes.contains(url.scheme())) {
         TRACE() << "Scheme not allowed:" << url.scheme();
         return true;
@@ -236,8 +246,11 @@ public:
     void start();
 
 private Q_SLOTS:
+    void onSslErrors(QNetworkReply *reply, const QList<QSslError> &errors);
     void onUrlChanged(const QUrl &url);
+    void onLoadProgress();
     void onLoadFinished(bool ok);
+    void onFailTimer();
     void onFinished();
     void startProgress();
     void stopProgress();
@@ -267,6 +280,7 @@ private:
     QStackedLayout *m_webViewLayout;
     WebView *m_webView;
     AnimationLabel *m_animationLabel;
+    HttpWarning *m_httpWarning;
     QUrl finalUrl;
     QUrl responseUrl;
     QString m_host;
@@ -277,6 +291,8 @@ private:
     QString m_username;
     QString m_password;
     int m_loginCount;
+    bool m_ignoreSslErrors;
+    QTimer m_failTimer;
 };
 
 } // namespace
@@ -288,9 +304,15 @@ BrowserRequestPrivate::BrowserRequestPrivate(BrowserRequest *request):
     m_webViewLayout(0),
     m_webView(0),
     m_animationLabel(0),
+    m_httpWarning(0),
     m_settings(0),
-    m_loginCount(0)
+    m_loginCount(0),
+    m_ignoreSslErrors(false)
 {
+    m_failTimer.setSingleShot(true);
+    m_failTimer.setInterval(3000);
+    QObject::connect(&m_failTimer, SIGNAL(timeout()),
+                     this, SLOT(onFailTimer()));
 }
 
 BrowserRequestPrivate::~BrowserRequestPrivate()
@@ -298,11 +320,21 @@ BrowserRequestPrivate::~BrowserRequestPrivate()
     delete m_dialog;
 }
 
+void BrowserRequestPrivate::onSslErrors(QNetworkReply *reply,
+                                        const QList<QSslError> &errors)
+{
+    TRACE() << errors;
+    if (m_ignoreSslErrors) {
+        reply->ignoreSslErrors();
+    }
+}
+
 void BrowserRequestPrivate::onUrlChanged(const QUrl &url)
 {
     Q_Q(BrowserRequest);
 
     TRACE() << "Url changed:" << url;
+    m_failTimer.stop();
 
     if (url.host() == finalUrl.host() &&
         url.path() == finalUrl.path()) {
@@ -317,6 +349,12 @@ void BrowserRequestPrivate::onUrlChanged(const QUrl &url)
     }
 
     setupViewForUrl(url);
+    m_httpWarning->setVisible(url.scheme() == "http");
+}
+
+void BrowserRequestPrivate::onLoadProgress()
+{
+    m_failTimer.stop();
 }
 
 void BrowserRequestPrivate::onLoadFinished(bool ok)
@@ -324,7 +362,7 @@ void BrowserRequestPrivate::onLoadFinished(bool ok)
     TRACE() << "Load finished" << ok;
 
     if (!ok) {
-        notifyLoadFailed();
+        m_failTimer.start();
         return;
     }
 
@@ -345,6 +383,11 @@ void BrowserRequestPrivate::onLoadFinished(bool ok)
     }
 }
 
+void BrowserRequestPrivate::onFailTimer()
+{
+    notifyLoadFailed();
+}
+
 void BrowserRequestPrivate::addBrowserCookies(CookieJar *cookieJar)
 {
     Q_Q(BrowserRequest);
@@ -373,16 +416,7 @@ void BrowserRequestPrivate::addBrowserCookies(CookieJar *cookieJar)
     QList<QNetworkCookie> cookies;
     RawCookies::const_iterator i;
     for (i = rawCookies.constBegin(); i != rawCookies.constEnd(); i++) {
-        const QString &host = i.key();
-        QStringList cookieList = i.value().split(";");
-        foreach (const QString &cookieSpec, cookieList) {
-            int semicolon = cookieSpec.indexOf("=");
-            QNetworkCookie cookie(cookieSpec.left(semicolon).toUtf8(),
-                                  cookieSpec.mid(semicolon + 1).toUtf8());
-            cookie.setDomain(host);
-            cookie.setPath("/");
-            cookies.append(cookie);
-        }
+        cookies.append(QNetworkCookie::parseCookies(i.value().toUtf8()));
     }
 
     TRACE() << "cookies:" << cookies;
@@ -392,13 +426,13 @@ void BrowserRequestPrivate::addBrowserCookies(CookieJar *cookieJar)
 void BrowserRequestPrivate::startProgress()
 {
     m_animationLabel->start();
-    m_webViewLayout->setCurrentWidget(m_animationLabel);
+    m_webViewLayout->setCurrentIndex(1);
 }
 
 void BrowserRequestPrivate::stopProgress()
 {
     m_animationLabel->stop();
-    m_webViewLayout->setCurrentWidget(m_webView);
+    m_webViewLayout->setCurrentIndex(0);
 }
 
 QWidget *BrowserRequestPrivate::buildWebViewPage(const QVariantMap &params)
@@ -412,6 +446,9 @@ QWidget *BrowserRequestPrivate::buildWebViewPage(const QVariantMap &params)
     WebPage *page = new WebPage(this);
     QObject::connect(page, SIGNAL(contentsChanged()),
                      this, SLOT(onContentsChanged()));
+    QObject::connect(page->networkAccessManager(),
+                     SIGNAL(sslErrors(QNetworkReply*,const QList<QSslError> &)),
+                     this, SLOT(onSslErrors(QNetworkReply*,const QList<QSslError> &)));
     m_webView->setPage(page);
 
     /* The following couple of lines serve to instruct the QWebPage not to load
@@ -440,13 +477,27 @@ QWidget *BrowserRequestPrivate::buildWebViewPage(const QVariantMap &params)
         page->setAllowedSchemes(QStringList("https"));
     }
 
+    m_ignoreSslErrors = clientData.value(keyIgnoreSslErrors, false).toBool();
+
     QUrl url(params.value(SSOUI_KEY_OPENURL).toString());
     setupViewForUrl(url);
     QObject::connect(m_webView, SIGNAL(urlChanged(const QUrl&)),
                      this, SLOT(onUrlChanged(const QUrl&)));
+    QObject::connect(m_webView, SIGNAL(loadProgress(int)),
+                     this, SLOT(onLoadProgress()));
     QObject::connect(m_webView, SIGNAL(loadFinished(bool)),
                      this, SLOT(onLoadFinished(bool)));
-    m_webViewLayout->addWidget(m_webView);
+    QWidget *webViewContainer = new QWidget;
+    QVBoxLayout *vLayout = new QVBoxLayout;
+    vLayout->setSpacing(0);
+    webViewContainer->setLayout(vLayout);
+    vLayout->addWidget(m_webView);
+
+    m_httpWarning = new HttpWarning;
+    m_httpWarning->setVisible(false);
+    vLayout->addWidget(m_httpWarning);
+
+    m_webViewLayout->addWidget(webViewContainer);
 
     m_animationLabel = new AnimationLabel(":/spinner-26.gif", 0);
     QObject::connect(m_webView, SIGNAL(loadStarted()),
@@ -687,7 +738,9 @@ QWebElement BrowserRequestPrivate::initializeField(const QString &settingsKey,
         const QVariantMap &params = q->parameters();
         if (!paramKey.isEmpty() && params.contains(paramKey)) {
             QString value = params.value(paramKey).toString();
-            element.setAttribute("value", value);
+            if (!value.isEmpty()) {
+                element.setAttribute("value", value);
+            }
         }
     } else {
         BLAME() << "Couldn't find element:" << selector;
